/// <reference types="node" />
import { BaseTx, ITransaction } from './trxTypes/BaseTx';
/**
 * Generic Abstract Wallet.
 */
export declare abstract class GenericWallet {
    /**
     * Verifies message. Returns true or throws error if verification fails
     * @param {string | Buffer} message
     * @param {string | Buffer} signature
     * @param {string | Buffer} pubKey
     * @returns {true}
     */
    static verifyMessage(message: string | Buffer, signature: string | Buffer, pubKey: string | Buffer): true;
    protected _privKey: string;
    protected _publicKey: string;
    protected _address: string;
    constructor(secret: string);
    signMessage(message: string | Buffer): any;
    /**
     * Signs a transaction
     * @param {ITransaction<T> | BaseTx<T>} tx
     * @param {GenericWallet} coSign optional cosigning wallet.
     * @returns {ITransaction<T>} signed transaction
     */
    signTransaction<T>(tx: ITransaction<T> | BaseTx<T>, coSign?: GenericWallet): ITransaction<T>;
    /**
     * Returns signature of provided transaction without modifiying the original tx.
     * @return {string} signature buffer.
     */
    getSignatureOfTransaction(tx: ITransaction<any> | BaseTx<any>): string;
    /**
     * Creates key pair from secret string
     * @param {string} secret
     */
    protected createKeyPair(secret: string): void;
    abstract readonly address: any;
    protected abstract deriveAddress(): any;
    /**
     * @returns {string} privateKey
     */
    readonly privKey: string;
    /**
     * @returns {string} publicKey
     */
    readonly publicKey: string;
    protected abstract readonly addressOptions: {
        suffixLength: number;
        suffix: string;
    };
}
