# glob-object [![NPM version](https://img.shields.io/npm/v/glob-object.svg?style=flat)](https://www.npmjs.com/package/glob-object) [![NPM monthly downloads](https://img.shields.io/npm/dm/glob-object.svg?style=flat)](https://npmjs.org/package/glob-object) [![NPM total downloads](https://img.shields.io/npm/dt/glob-object.svg?style=flat)](https://npmjs.org/package/glob-object) [![Linux Build Status](https://img.shields.io/travis/jonschlinkert/glob-object.svg?style=flat&label=Travis)](https://travis-ci.org/jonschlinkert/glob-object)

> Filter an object using glob patterns and dot notation.

## Install

Install with [npm](https://www.npmjs.com/):

```sh
$ npm install --save glob-object
```

## Usage

```js
var globObject = require('glob-object');

globObject('a.*.f', {a: {b: {c: 'd'}, e: {f: 'g'}}});
//=> { a: { e: { f: 'g' } } }
```

## Examples

Given the following object:

```js
var obj = {
  a: {
    b: {
      c: 'd',
      e: 'f',
      g: 'h',
      i: {j: 'k'},
      l: {g: 'k'}
    },
    i: 'j'
  }
};
```

## match properties using wildcards

```js
globObject('*', obj);
//=> obj (matches all keys)
```

## match properties using braces

```js
globObject('a.*.{c,e}', obj);
//=> {a: {b: {c: 'd', e: 'f'}}}
```

## match a nested property using a wildcard

A single star will match one level of the object:

```js
globObject('a.*.g', obj);
//=> {a: {b: {g: 'h'}}}
```

## match deep properties using globstars

A double star will match to any depth (note that the single star in the previous example did not match `a.b.l.g`):

```js
globObject('a.**.g', obj);
//=> {a: {b: {g: 'h', l: {g: 'k'}}}}
```

## About

### Related projects

* [expand-object](https://www.npmjs.com/package/expand-object): Expand a string into a JavaScript object using a simple notation. Use the CLI or… [more](https://github.com/jonschlinkert/expand-object) | [homepage](https://github.com/jonschlinkert/expand-object "Expand a string into a JavaScript object using a simple notation. Use the CLI or as a node.js lib.")
* [get-value](https://www.npmjs.com/package/get-value): Use property paths (`a.b.c`) to get a nested value from an object. | [homepage](https://github.com/jonschlinkert/get-value "Use property paths (`a.b.c`) to get a nested value from an object.")
* [has-value](https://www.npmjs.com/package/has-value): Returns true if a value exists, false if empty. Works with deeply nested values using… [more](https://github.com/jonschlinkert/has-value) | [homepage](https://github.com/jonschlinkert/has-value "Returns true if a value exists, false if empty. Works with deeply nested values using object paths.")
* [set-value](https://www.npmjs.com/package/set-value): Create nested values and any intermediaries using dot notation (`'a.b.c'`) paths. | [homepage](https://github.com/jonschlinkert/set-value "Create nested values and any intermediaries using dot notation (`'a.b.c'`) paths.")
* [stringify-keys](https://www.npmjs.com/package/stringify-keys): Build an array of key paths from an object. | [homepage](https://github.com/doowb/stringify-keys "Build an array of key paths from an object.")

### Contributing

Pull requests and stars are always welcome. For bugs and feature requests, [please create an issue](../../issues/new).

### Building docs

_(This project's readme.md is generated by [verb](https://github.com/verbose/verb-generate-readme), please don't edit the readme directly. Any changes to the readme must be made in the [.verb.md](.verb.md) readme template.)_

To generate the readme, run the following command:

```sh
$ npm install -g verbose/verb#dev verb-generate-readme && verb
```

### Running tests

Running and reviewing unit tests is a great way to get familiarized with a library and its API. You can install dependencies and run tests with the following command:

```sh
$ npm install && npm test
```

### Author

**Jon Schlinkert**

* [github/jonschlinkert](https://github.com/jonschlinkert)
* [twitter/jonschlinkert](https://twitter.com/jonschlinkert)

### License

Copyright © 2017, [Jon Schlinkert](https://github.com/jonschlinkert).
Released under the [MIT License](LICENSE).

***

_This file was generated by [verb-generate-readme](https://github.com/verbose/verb-generate-readme), v0.6.0, on June 19, 2017._